@@ -253,7 +253,22 @@ int main(int argc, char* argv[])
             }
         }
     }
-
+/*
+    /// saphireは前面にあるか？ ///
+    BOOL saphire_fg;
+    pid_t pgrp =  tcgetpgrp(0);
+    if(pgrp < 0) {
+        perror("tcgetpgrp");
+        exit(1);
+    }
+    pid_t pgrp2 = getpgrp();
+    if(pgrp == pgrp2) {
+        saphire_fg = TRUE;
+    }
+    else {
+        saphire_fg = FALSE;
+    }
+*/
     /// 引数のファイル名がディレクトリじゃなければスクリプトと判断する ///
     if(vector_size(argv2) > 0) {
         char* file_name = string_c_str(vector_item(argv2, 0));
@@ -267,7 +282,8 @@ int main(int argc, char* argv[])
         if(!S_ISDIR(stat_.st_mode)) {
             mcurses_init(code);
             saphire_set_signal();
-            saphire_init(kATConsoleApp, FALSE, runtime_script, FALSE);
+            saphire_init(kATOptC, FALSE, runtime_script, FALSE);
+            //saphire_init(kATConsoleApp, FALSE, runtime_script, FALSE);
 
             saphire_set_global_var("SCRIPT_FILE_NAME", file_name);
 
