@@ -37,7 +37,6 @@
 //////////////////////////////////////////////////////////////////////
 // static variable
 //////////////////////////////////////////////////////////////////////
-static vector_obj* gEnvirons = NULL;             // 環境変数補完候補
 static vector_obj* files = NULL;
 
 //////////////////////////////////////////////////////////////////////
@@ -549,24 +548,162 @@ char* readline_program_completion(const char* text, int stat)
     return NULL;
 }
 
+static vector_obj* gCompList;               // 環境変数補完候補
+
+static void readline_env_completion_make_completion_list()
+{
+    int i;
+    for(i=0; i<vector_size(gCompList); i++) {
+        string_delete(vector_item(gCompList, i));
+    }
+    vector_clear(gCompList);
+
+    /// 環境変数 ///
+#if !HAVE_DECL_ENVIRON
+    extern char **environ;
+
+    char** p;
+    for(p = environ; *p; p++) {
+        char env_name[256];
+
+        char* p2 = env_name;
+        char* p3 = *p;
+
+        *p2++ = '$';
+        while(*p3 != '=') {
+            *p2++ = *p3++;
+        }
+        *p2 = 0;
+
+        vector_add(gCompList, STRING_NEW(env_name));
+    }
+#endif
+
+    hash_it* it = hash_loop_begin(gGlobals);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[256];
+        strcpy(env_name, "$");
+        strcat(env_name, key);
+
+        vector_add(gCompList, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    it = hash_loop_begin(gArrays);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[256];
+        strcpy(env_name, "$");
+        strcat(env_name, key);
+
+        vector_add(gCompList, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    it = hash_loop_begin(gHashs);
+    while(it) {
+        char* key = hash_loop_key(it);
+
+        char env_name[256];
+        strcpy(env_name, "$");
+        strcat(env_name, key);
+
+        vector_add(gCompList, STRING_NEW(env_name));
+
+        it = hash_loop_next(it);
+    }
+
+    (void)vector_sort(gCompList, name_sort);
+}
+
 char* readline_env_completion(const char* text, int stat)
 {
     static int index, wordlen;
+    static hash_it* git;
+    static hash_it* ait;
+    static hash_it* hit;
 
     if(stat == 0) {
         wordlen = strlen(text);
         index = 0;
+    
+        readline_env_completion_make_completion_list();
     }
-
-    if(gEnvirons) {
-        while(index < vector_size(gEnvirons)) {
-            char* env = string_c_str(vector_item(gEnvirons, index));
+    while(1) {
+        if(index < vector_size(gCompList)) {
+            char* env = string_c_str(vector_item(gCompList, index));
             index++;
-
-            if(!strncmp(text, env, wordlen)) {
+            if(strncmp(text, env, wordlen) == 0) {
                 return strdup(env);
             }
         }
+/*
+        else if(git != NULL) {
+            char* global = hash_loop_key(git);
+            git = hash_loop_next(git);
+
+            if(!strncmp(text2, global, wordlen2)) {
+                char* str = malloc(strlen(global) + 1 +dollar_num +atmark_num);
+                char * p = str;
+                int i;
+                for(i=0; i<dollar_num; i++) {
+                    *p++ = '$';
+                }
+                for(i=0; i<atmark_num; i++) {
+                    *p++ = '@';
+                }
+                strcpy(p, global);
+
+                return str;
+            }
+        }
+        else if(ait != NULL) {
+            char* array = hash_loop_key(ait);
+            ait = hash_loop_next(ait);
+
+            if(!strncmp(text2, array, wordlen2)) {
+                char* str = malloc(strlen(array) + 1 +dollar_num +atmark_num);
+                char * p = str;
+                int i;
+                for(i=0; i<dollar_num; i++) {
+                    *p++ = '$';
+                }
+                for(i=0; i<atmark_num; i++) {
+                    *p++ = '@';
+                }
+                strcpy(p, array);
+
+                return str;
+            }
+        }
+        else if(hit != NULL) {
+            char* hash = hash_loop_key(hit);
+            hit = hash_loop_next(hit);
+
+            if(!strncmp(text2, hash, wordlen2)) {
+                char* str = malloc(strlen(hash) + 1 +dollar_num +atmark_num);
+                char * p = str;
+                int i;
+                for(i=0; i<dollar_num; i++) {
+                    *p++ = '$';
+                }
+                for(i=0; i<atmark_num; i++) {
+                    *p++ = '@';
+                }
+                strcpy(p, hash);
+
+                return str;
+            }
+        }
+*/
+        else {
+            break;
+        }
     }
 
     return NULL;
@@ -1465,47 +1602,16 @@ void readline_init()
     migemo_init();
 #endif
 
-#if !HAVE_DECL_ENVIRON
-    extern char **environ;
-
-    gEnvirons = VECTOR_NEW(5);
-    char** p;
-    for(p = environ; *p; p++) {
-        char env_name[256];
-
-        char* p2 = env_name;
-        char* p3 = *p;
-
-        *p2++ = '$';
-        while(*p3 != '=') {
-            *p2++ = *p3++;
-        }
-
-        char* env = getenv(*p);
-        struct stat estat;
-        if(stat(env, &estat) >= 0) {
-            if(S_ISDIR(estat.st_mode)) {
-                *p2++ = '/';
-            }
-        }
-        *p2 = 0;
-
-        vector_add(gEnvirons, STRING_NEW(env_name));
-    }
-
-    (void)vector_sort(gEnvirons, name_sort);
-#endif
+    gCompList = VECTOR_NEW(100);
 }
 
 void readline_final()
 {
-    if(gEnvirons) {
-        int i;
-        for(i=0; i<vector_size(gEnvirons); i++) {
-            string_delete(vector_item(gEnvirons, i));
-        }
-        vector_delete(gEnvirons);
+    int i;
+    for(i=0; i<vector_size(gCompList); i++) {
+        string_delete(vector_item(gCompList, i));
     }
+    vector_delete(gCompList);
     if(files) {
         int i;
         for(i=0; i<vector_size(files); i++) {
