@@ -42,7 +42,9 @@ sRFd* sRFd_new2(int fd, char* str);
 void sRFd_delete(sRFd* self);
 BOOL sRFd_close(sRFd* self);
 int sRFd_read_all(sRFd* self, string_obj* str);
+int sRFd_read_all_preserve(sRFd* self, string_obj* str);
 int sRFd_read_oneline(sRFd* self, string_obj* str);
+int sRFd_read_oneline_preserve(sRFd* self, string_obj* str);
 int sRFd_read_all_to_buffer(sRFd* self);
 
 extern enum eKanjiCode gKanjiCode;
@@ -67,7 +69,7 @@ enum eRuntimeScript { kRSNoRead, kRSSource, kRSObject };
     // kRSSource ソースファイルを読み込む
     // kRSObject オブジェクトファイルを読み込む
 
-void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script);
+void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script, BOOL run_user_runtimescript);
     // 初期化。saphire_shellなどを実行する前に必ず必要
     // アプリケーションの種別とジョブコントロールの有無とランタイムスクリプトをどうするかを読み込むかを設定してください
     // あと、必要な初期化は端末制御ライブラリsaphire_curses.h
@@ -100,7 +102,7 @@ int saphire_shell(char* command, char* title, sWFd* pipeout, sRFd* pipein, int p
     // saphire内でエラーが起こっているのでgErrMsgをどこかに
     // 出力してください。
     // verboseは実行しているコマンド名を表示するかどうか
-int saphire_shell3(string_obj* result, char* command, char* title);
+int saphire_shell3(string_obj* result, char* command, char* title, sRFd* pipein);
     // 文字列を読み込んで実行。コマンドの出力はresult
     // (saphire_string.hで定義されているstring_obj*文字列型)に入る
     // 戻り値はプログラムの終了コードだが、0以下だとsaphire内でエラーが
@@ -151,6 +153,17 @@ void saphire_kill_all_jobs();
 
 extern vector_obj* gPrograms;               // 補完候補
 
+extern hash_obj* gGlobals;
+    // グローバル変数 キーは変数名 内容はstring_obj*
+extern hash_obj* gArrays;
+    // グローバル変数の配列 キーは変数名
+    // 内容は配列(vector_obj*) 配列の中にはstring_obj*
+    // が入っている
+extern hash_obj* gHashs;
+    // グローバル変数の配列 キーは変数名
+    // 内容はハッシュ(hash_obj*) ハッシュの中にはstring_obj*
+    // が入っている
+
 void saphire_rehash();
     // プログラム補完候補の一覧の再読み込み
     // $PATHから読み込む
@@ -199,10 +212,14 @@ void saphire_delete_local_var(char* name);
 char* saphire_get_global_var(char* name);
     // グローバル変数を返す
     // 無ければNULLが返る
+void saphire_set_global_var(char* name, char* value);
+    // グローバル変数を設定する
 
 vector_obj* saphire_get_array(char* name);
     // 配列を返す string_obj*(文字列型)が入ったvector_obj*(動的配列)
     // 無ければNULLが返る
+void saphire_set_array(char* name, vector_obj* array);
+    // 配列を設定する
 
 hash_obj* saphire_get_hash(char* name);
     // ハッシュを返す string_obj*(文字列型)が入ったhash_obj*(ハッシュ)
@@ -215,7 +232,7 @@ typedef struct
 
 sStatments* sStatments_new();
 void sStatments_delete(sStatments* self);
-int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL finaly_exec, BOOL in_loop);
+int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL finaly_exec, BOOL in_loop, BOOL* return_);
     // saphire_compile2でコンパイルしたコンパイル結果を実行する。
     // verboseは実行する前に実行するコマンド名を表示するかどうか
 BOOL saphire_compile2(char* cmdline, char* fname, sStatments* statments);
@@ -251,12 +268,26 @@ BOOL statment_tree_internal_commands_write_nextout(sWFd* nextout, char* str);
 // 戻り値 1 --> EOF
 int statment_tree_internal_commands_read_nextin(sRFd* nextin, string_obj* str);
 
+// 出力をCTRL-Cで止めたいので、この関数を経由して読み込む
+// 戻り値 -1 --> CTRL-Cで止められた
+// 戻り値 0 --> 正常終了
+// 戻り値 1 --> EOF
+// 読み込んでもバッファを読み込んだ分のバッファを消さない
+int statment_tree_internal_commands_read_nextin_preserve(sRFd* nextin, string_obj* str);
+
 // 出力をCTRL-Cで止めたいので、この関数を経由して読み込む
 // 戻り値 -1 --> CTRL-Cで止められた
 // 戻り値 0 --> 正常終了
 // 戻り値 1 --> EOF
 int statment_tree_internal_commands_read_nextin_oneline(sRFd* nextin, string_obj* str);
 
+// 出力をCTRL-Cで止めたいので、この関数を経由して読み込む
+// 戻り値 -1 --> CTRL-Cで止められた
+// 戻り値 0 --> 正常終了
+// 戻り値 1 --> EOF
+// 読み込んでも読み込んだ分のバッファを消さない
+int statment_tree_internal_commands_read_nextin_oneline_preserve(sRFd* nextin, string_obj* str);
+
 
 #endif
 
