@@ -97,9 +97,18 @@ extern hash_obj* gReadBuffers;
     //  sRFd*
     // readѤΥեΥ꡼ɥХåե
 
-vector_obj* gGlobalPipes;
+extern vector_obj* gGlobalPipes;
     // å֥륰Хѥפɤ߹ߤǡ
 
+extern vector_obj* gQGlobalPipes;
+    // 塼Хѥפɤ߹ߤǡ
+
+extern string_obj* gGlobalPipe;
+    // Хѥפɤ߹ߤǡ
+
+extern string_obj* gGlobalPipeNum[10];
+    // ʥСХѥפɤ߹ߤǡ
+
 extern hash_obj* gRegexs;
     // ɽΥå
     // ɽʸ
@@ -364,8 +373,10 @@ typedef struct
 
     BOOL mBackground;        // Хå饦ɤǼ¹
 
-    int mGlobalPipeIn;   // ХѥפϤ뤫 1:  2: å֥
-    int mGlobalPipeOut;  // ХѥפνϤ뤫 1:  2: å֥
+    int mGlobalPipeIn;   // ХѥפϤ뤫 1:  2: å֥ 3:塼 4: ʥС
+    int mGlobalPipeOut;  // ХѥפνϤ뤫 1:  2: å֥ 3:塼 4:ʥС
+    int mGlobalPipeInNum; // ʥСХѥפֹ
+    int mGlobalPipeOutNum; // ʥСХѥפֹ
 } sStatment;
 
 #ifdef MDEBUG
@@ -456,7 +467,7 @@ BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* s
 ////////////////////////////////////////////////////////////
 /// from saphire_parser.c
 ////////////////////////////////////////////////////////////
-BOOL expand_env(char* p, string_obj* out, char* sname, int* line);
+BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line);
 BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
 BOOL read_expand_brace_expansion(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);
 BOOL parse_number(char* p, vector_obj* numbers, char* sname, int line);
