@@ -287,7 +287,6 @@ void string_pomch(string_obj* str)
 // gFileHandlesにファイルハンドルがあれば削除
 BOOL xclose(int fd) 
 {
-//printf("xclose %d\n", fd);
     char buf[32];
     snprintf(buf, 32, "%d", fd);
 
@@ -3193,6 +3192,172 @@ int sRFd_read_oneline_preserve(sRFd* self, string_obj* str)
     }
 }
 
+// 0: 正常終了 1: EOF -1:エラー
+int sRFd_read_oneline_preserve_num(sRFd* self, string_obj* str, int line_num)
+{
+    if(self->mFd == -1) {
+        string_obj* buffer = self->mBuffer;
+        if(gKitutukiSigInt) {
+            gKitutukiSigInt = FALSE;
+            return -1;
+        }
+
+        /// バッファーが無い
+        if(strcmp(string_c_str(buffer), "") == 0) {
+            return 1;
+        }
+
+        /// 取ったバッファからn行を得て返す ///
+        char *p;
+        int lf_char_size;
+        if(gLineField == kLF) {
+            int i;
+            p = strstr(string_c_str(buffer), "\n"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+1, "\n"); 
+            }
+            lf_char_size = 1;
+        }
+        else if(gLineField == kCRLF) {
+            int i;
+            p = strstr(string_c_str(buffer), "\r\n"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+2, "\r\n"); 
+            }
+            lf_char_size = 2;
+        }
+        else {
+            int i;
+            p = strstr(string_c_str(buffer), "\r"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+1, "\r"); 
+            }
+            lf_char_size = 1;
+        }
+
+        /// 改行がある
+        if(p) {
+            int n = p - string_c_str(buffer);
+            string_push_back3(str, string_c_str(buffer), n + lf_char_size);
+            return 0;
+        }
+        /// 改行が無い
+        else {
+            string_push_back(str, string_c_str(buffer));
+
+            return 0;
+        }
+    }
+    else {
+        while(1) {
+            /// 改行がn個含まれていればbreakする ///
+            if(gLineField == kCR) {
+                char* p = strstr(string_c_str(self->mBuffer), "\r");
+                int n = 0;
+                while(p != NULL && n<line_num) {
+                    p = strstr(p+1, "\r");
+                    n++;
+                }
+                if(n == line_num) {
+                    break;
+                }
+            }
+            else if(gLineField == kLF) {
+                char* p = strstr(string_c_str(self->mBuffer), "\n");
+                int n = 0;
+                while(p != NULL && n<line_num) {
+                    p = strstr(p+1, "\n");
+                    n++;
+                }
+                if(n == line_num) {
+                    break;
+                }
+            }
+            else {
+                char* p = strstr(string_c_str(self->mBuffer), "\r\n");
+                int n = 0;
+                while(p != NULL && n<line_num) 
+                {
+                    p = strstr(p+2, "\r\n");
+                    n++;
+                }
+                if(n == line_num) {
+                    break;
+                }
+            }
+
+            char buf[BUFSIZ+1];
+
+            int r = read(self->mFd, buf, BUFSIZ);
+
+            if(r < 0) {
+                return -1;
+            }
+            if(gKitutukiSigInt) {
+                gKitutukiSigInt = FALSE;
+                return -1;
+            }
+            buf[r] = 0;
+
+            /// EOF ///
+            if(r == 0) {
+                break;
+            }
+            else {
+                string_push_back(self->mBuffer, buf);
+            }
+        }
+
+        /// バッファーが無い
+        if(strcmp(string_c_str(self->mBuffer), "") == 0) {
+            return 1;
+        }
+
+        /// 取ったバッファからn行を得て返す ///
+        char *p;
+        int lf_char_size;
+        if(gLineField == kLF) {
+            int i;
+            p = strstr(string_c_str(self->mBuffer), "\n"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+1, "\n"); 
+            }
+            lf_char_size = 1;
+        }
+        else if(gLineField == kCRLF) {
+            int i;
+            p = strstr(string_c_str(self->mBuffer), "\r\n"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+2, "\r\n"); 
+            }
+            lf_char_size = 2;
+        }
+        else {
+            int i;
+            p = strstr(string_c_str(self->mBuffer), "\r"); 
+            for(i=0; i<line_num-1 && p != NULL; i++) {
+                p = strstr(p+1, "\r"); 
+            }
+            lf_char_size = 1;
+        }
+
+        /// 改行がある
+        if(p) {
+            int n = p - string_c_str(self->mBuffer);
+            string_push_back3(str, string_c_str(self->mBuffer), n + lf_char_size);
+
+            return 0;
+        }
+        /// 改行が無い
+        else {
+            string_push_back(str, string_c_str(self->mBuffer));
+
+            return 0;
+        }
+    }
+}
+
+
 sWFd* sWFd_new(int fd)
 {
     sWFd* self = MALLOC(sizeof(sWFd));
